<p align="center"><img src="Resources/repo-banner.png" alt="Fluid Interfaces Title Graphic"></p>

### Check out the accompanying blog post for more details: [Building Fluid Interfaces](https://medium.com/@nathangitter/building-fluid-interfaces-ios-swift-9732bb934bf5)

⭐️ If you found this repo helpful, please star it. Thanks! ⭐️

### Background

At WWDC 2018, Apple designers gave a talk titled "Designing Fluid Interfaces", explaining the design theory behind the gesture-based interface of iPhone X. (The presentation is available here: [Designing Fluid Interfaces](https://developer.apple.com/videos/play/wwdc2018/803/))

It was an outstanding talk, inspiring designers and developers to think about animated interfaces in a new way. While some technical guidance was provided, most code-level implementation details were ommitted.

The goal with this project is to bridge the gap between inspiration and implementation.

### Installation

Download or clone the repo and open the `FluidInterfaces.xcodeproj` file with Xcode 10.

If you are using Xcode 9, switch to the branch named "Xcode9".

### Interfaces

[Calculator Button](#calculator-button)

[Spring Animations](#spring-animations)

[Flashlight Button](#flashlight-button)

[Rubberbanding](#rubberbanding)

[Acceleration Pausing](#acceleration-pausing)

[Rewarding Momentum](#rewarding-momentum)

[FaceTime PiP](#facetime-pip)

[Rotation](#rotation)

## Calculator Button

A button that mimics the behavior of buttons in the iOS Calculator app.

<img src="Resources/calcdemo.gif" alt="Calculator Button Demo">

[⚙️ CalculatorButton.swift](FluidInterfaces/FluidInterfaces/CalculatorButton.swift)

[📺 Designing Fluid Interfaces 50:13](https://developer.apple.com/videos/play/wwdc2018/803/?time=3013)

## Spring Animations

A demo showing the ability to define a spring animation with "design-friendly" parameters.

<img src="Resources/springdemo.gif" alt="Spring Demo">

[⚙️ Spring.swift](FluidInterfaces/FluidInterfaces/Spring.swift)

[📺 Designing Fluid Interfaces 31:40](https://developer.apple.com/videos/play/wwdc2018/803/?time=1900)

## Flashlight Button

A button that mimics the behavior of the flashlight button on the lockscreen of iPhone X.

<img src="Resources/flashdemo.gif" alt="Flashlight Button Demo">

[⚙️ FlashlightButton.swift](FluidInterfaces/FluidInterfaces/FlashlightButton.swift)

[📺 Designing Fluid Interfaces 37:59](https://developer.apple.com/videos/play/wwdc2018/803/?time=2279)

## Rubberbanding

A demo showing how to implement rubberbanding.

<img src="Resources/rubberdemo.gif" alt="Rubberbanding Demo">

[⚙️ Rubberbanding.swift](FluidInterfaces/FluidInterfaces/Rubberbanding.swift)

[📺 Designing Fluid Interfaces 17:01](https://developer.apple.com/videos/play/wwdc2018/803/?time=1021)

## Acceleration Pausing

A demo of using a gesture's acceleration to quickly detect when the user's motion has paused.

<img src="Resources/accelerationdemo.gif" alt="Acceleration Pausing Demo">

[⚙️ Acceleration.swift](FluidInterfaces/FluidInterfaces/Acceleration.swift)

[📺 Designing Fluid Interfaces 10:40](https://developer.apple.com/videos/play/wwdc2018/803/?time=640)

## Rewarding Momentum

A drawer with an open and closed state which has bounciness dependent on the velocity of the gesture.

<img src="Resources/momentumdemo.gif" alt="Rewarding Momentum Demo">

[⚙️ Momentum.swift](FluidInterfaces/FluidInterfaces/Momentum.swift)

[📺 Designing Fluid Interfaces 36:48](https://developer.apple.com/videos/play/wwdc2018/803/?time=2208)

## FaceTime PiP

A re-creation of the picture-in-picture UI of the iOS FaceTime app.

<img src="Resources/pipdemo.gif" alt="FaceTime PiP Demo">

[⚙️ Pip.swift](FluidInterfaces/FluidInterfaces/Pip.swift)

[📺 Designing Fluid Interfaces 41:56](https://developer.apple.com/videos/play/wwdc2018/803/?time=2516)

## Rotation

A demo showing how the concepts from the PiP interface can apply to other animations.

<img src="Resources/rotationdemo.gif" alt="Rotation Demo">

[⚙️ Rotation.swift](FluidInterfaces/FluidInterfaces/Rotation.swift)

[📺 Designing Fluid Interfaces 47:25](https://developer.apple.com/videos/play/wwdc2018/803/?time=2845)

## Author
You can find me on Twitter [@nathangitter](https://twitter.com/nathangitter)

## Copyright
© 2018 Nathan Gitter
